var config;

var legendTask;
var legendComplete = {};

// this is for IE8
(function (fallback) {

    fallback = fallback || function () { };

    // function to trap most of the console functions from the FireBug Console API.
    var trap = function () {
        // create an Array from the arguments Object
        var args = Array.prototype.slice.call(arguments);
        // console.raw captures the raw args, without converting toString
        console.raw.push(args);
        var message = args.join(' ');
        console.messages.push(message);
        fallback(message);
    };

    // redefine console
    if (typeof console === 'undefined') {
        console = {
            messages: [],
            raw: [],
            dump: function() { return console.messages.join('\n'); },
            log: trap,
            debug: trap,
            info: trap,
            warn: trap,
            error: trap,
            assert: trap,
            clear: function() {
                  console.messages.length = 0;
                  console.raw.length = 0 ;
            },
            dir: trap,
            dirxml: trap,
            trace: trap,
            group: trap,
            groupCollapsed: trap,
            groupEnd: trap,
            time: trap,
            timeEnd: trap,
            timeStamp: trap,
            profile: trap,
            profileEnd: trap,
            count: trap,
            exception: trap,
            table: trap
        };
    }

})(null);

var waitReady = function(){
    try {
        if ( !OpenLayers.WPSProcess || !CWM || !OpenLayers.Console) {
            throw 'not yet';
        }
    }
    catch (e) {
        setTimeout( waitReady, 500);
        return;
    }
    setupDocument();
};
waitReady();

function setupDocument() {
    config = getConfig();
    config.map.showLoadingSpinner = false;

    // create main map
    var mb1 = new CWM.MapBuilder();
    mb1.addMapConfig( config.map );
    mb1.addOverlayConfig( config.overlay );
    var map1 = mb1.createMap( "map" );

    mb1.map.olmap.addControl( new OpenLayers.Control.ScaleLine( {
        geodesic: true
    } ) );

    if ( config.showGraticule ) {
        var glayer = new OpenLayers.Layer.Vector( CWM.LayerName.GRATICULE, {
            styleMap: CWM.Style.GRATICULE,
            visibility: true,
            displayInLayerSwitcher: false
        } );
        mb1.map.olmap.addLayer( glayer );

        var graticuleCtl = new OpenLayers.Control.GraticuleEx( {
            id: "GraticuleControl",
            labelled: true,
            labelFormat: 'd*',
            labelLonYOffset: 20,
            numPoints: 2,
            intervalHeightFactor: 0.5,
            // BCGS grid
            intervals: [ 8, 2, 0.2 ],
            // MTO grid
            //intervals: [ 8, 2, 0.5,  0.125,              0.0125,               0.00625  ],
            //intervalHeights: [ 4, 1, 0.25, 0.0833333333333333, 0.00833333333333333,  0.0041666666666667 ],
            targetSize: 600,
            visible: false,
            gratLayer: glayer
        } );
        mb1.map.olmap.addControl( graticuleCtl );
    }

    mb1.map.olmap.setCenter( config.center, config.zoom );

    // set overview image
    var overview = {
        image: "bcroads-overview.png",
        dimensions: {
            width: 1200,
            height: 950
        },
        extent: [
            -139.28314926554515, 48.03939402151377, -112.91596176921718, 60.167375109092184
        ]
    };

    var ov = $('#overview');
    var ovw = ov.width();
    var ovh = ov.height();
    var ovr = ovw / ovh;
    var imr = overview.dimensions.width / overview.dimensions.height;
    var sf, l,t,w,h;
    var scale;

    var bounds = CWM.CRS.transformToMap( new OpenLayers.Bounds( overview.extent ));

    // size so that image is centered and fills frame
    if ( ovr > imr ) {
        // console.log('overview is wider than image');
        sf = ovw / overview.dimensions.width

        l = 0;
        w = ovw;

        h = overview.dimensions.height * sf;
        t = ( ovh - h ) / 2;

        scale = ovw / ( bounds.right - bounds.left );
    }
    else {
        // console.log('image is wider than overview ');
        sf = ovh / overview.dimensions.height

        w = overview.dimensions.width * sf;
        l = ( ovw - w ) / 2;

        t = 0;
        h = ovh;

        scale = ovh / ( bounds.bottom - bounds.top );
    }

    ov.append( $('<img>').attr('src',overview.image).css({
        position: 'relative',
        width: w+'px',
        height: h+'px',
        left: l+'px',
        top: t+'px'
    }))

    var mapExtent = mb1.map.olmap.getExtent();

    var left   = clamp( Math.round( scale * ( mapExtent.left - bounds.left ) ), 3, ovw);
    var bottom = clamp( Math.round( scale * ( bounds.top - mapExtent.bottom ) ), 1, ovh);
    var right  = clamp( Math.round( scale * ( mapExtent.right - bounds.left ) ), 1, ovw);
    var top    = clamp(Math.round( scale * ( bounds.top - mapExtent.top ) ), 3, ovh);
    var width  = clamp(right - left, 3, ovw);
    var height = clamp(bottom - top, 3, ovh);

    var overviewRect = $( '<div class="overview-extent">' ).appendTo( $( '#overview' ) );
    overviewRect.css( {
        left: left,
        top: top,
        width: width,
        height: height
    } )

    // create legend
    var div = $( '#legend' );

    var imageComplete = function (e) {
        // console.log(e.data);
        legendComplete[e.data] = true;
    };

    $.each( config.legend, function ( i, ovly ) {
        div.append( $('<div class="overlay-title">').text( ovly.title ) );

        $.each( ovly.legends, function ( j, leg ) {
            legendComplete[ j ] = false;

            if ( leg.url ) {
                if ( leg.title )
                    $('<div class="legend-title">').text( leg.title ).appendTo( div )

                leg.image = $('<img>')
                    .on({
                        load: imageComplete,
                        error: imageComplete,
                        readystatechange: imageComplete
                    },null,j)
                    .attr('src', leg.url )
                    .appendTo( div );
            }

            if ( leg.content ) {
                div.append( leg.content )
                legendComplete[ j ] = true
            }

            // console.log( 'request '+leg.url);
        });
    })

    legendTask = setTimeout( fitLegend, 200, 0 );

    // fill in labels
    $.each( config.labelMap, function ( key, val ) {
        var el = $( '#' + key );
        if ( el.length == 0 ) return;

        el.html( val );
    } )

    // draw vector features (AOI, markup, imported layers) on map
    var format = new OpenLayers.Format.GeoJSON();
    $.each( config.vectors, function ( i, v ) {
        var f = format.read( v.geometry );
        if ( f[ 0 ] && f[ 0 ].geometry )
            map1.markup.addGeometry( f[ 0 ].geometry, v.style );
    })

    // show popup balloon (if any)
    if ( config.popup ) {
        var popup = new OpenLayers.Popup.FramedCloud(
            "featurePopup",
            new OpenLayers.LonLat( config.popup.point ),
            new OpenLayers.Size( 200, 200 ),
            config.popup.markup,
            null,
            true
        );
        popup.autoSize = true;

        mb1.map.olmap.addPopup( popup );
    }

    // create scale block
    var scale = $( '#scale' );
    if ( scale ) {
        CWM.CoordinateFormat.LatLonDMS.degreeSuffix = '&deg;';
        CWM.CoordinateFormat.LatLonDMS.minuteSuffix = "'";
        CWM.CoordinateFormat.LatLonDMS.secondSuffix = '"';
        CWM.CoordinateFormat.LatLonDMS.numSecondsDigits = 0;

        var ll = CWM.CoordinateFormat.LatLonDMS.value( CWM.CRS.transformMapToGeo( new OpenLayers.LonLat( config.center ))).replace( ' ', '&nbsp;' );

        scale.append(
            "<table>"+
            "<tr><td><b>Center:</b></td><td>"+ll+"</td></tr>"+
            "<tr><td><b>Scale:</b></td><td>1&nbsp;:&nbsp;"+config.scale+"</td></tr>"+
            "<tr><td><b>SRS:</b></td><td>"+config.srs+"</td></tr>"+
            "<tr><td><b>UTM&nbsp;Zone:</b></td><td>"+config.utmZone+"</td></tr>"+
            "</table>"
        );
    }

    // wait for loading to finish and print
    var waitLoad = function ( count ) {
        var next = waitLoad;
        var loading = $.grep( mb1.map.olmap.layers, function ( l ) {
            return l.loading
        } );

        if ( count > 50 || ( legendTask == null && loading.length == 0 ) ) {
            next = printDocument;
        }

        setTimeout( next, 250, count + 1 )
    }
    waitLoad( 0 )
};

function clamp(i, min, max) {
	if (i < min) return min;
	if (i > max) return max;
	return i;
}
function fitLegend( count ) {
    var images = $.map( config.legend, function ( ovly ) {
        return $.map( ovly.legends, function ( leg ) {
            return leg.image
        })
    });

    var widths = $.map( images, function (im) { return $(im).width() } );

    var allComplete = $.map( legendComplete, function(v) { if (v) return v }).length;

    legendTask = null;
    if ( allComplete != images.length ) {
        // console.log( allComplete,images.length);
        if ( count < 200 )
            legendTask = setTimeout( fitLegend, 100, count+1 );
        return;
    }

    var max = Math.max.apply( this, widths );

    var container = $( '#legend' ).innerWidth() - 5;

    if ( max <= container ) return;

    var scale = container / max;
    // console.log(scale);

    $.each( images, function (i, im) {
        im.css( {
            width: (im.width() * scale) + "px",
            height: (im.height() * scale) + "px"
        })
    })
}

function printDocument() {
    document.close();
    if ( config.autoPrint ) {
        window.print();
        window.close();
    }
    else {
        console.log( 'ready to print' )
    }
}

function getConfig() {
    var key = window.location.search.substr( 1 );

    var json = window.localStorage.getItem( key );

    var cfg = JSON.parse( json );

    if ( cfg.autoPrint )
        window.localStorage.removeItem( key );

    return cfg;
}
